@@ -91,7 +91,7 @@ void                  kgx_settings_set_custom_size      (KgxSettings       *self
                                                          gboolean           maximised);
 gboolean              kgx_settings_get_audible_bell  (KgxSettings       *self);
 gboolean              kgx_settings_get_visual_bell   (KgxSettings       *self);
-PangoFontDescription *kgx_settings_get_custom_font      (KgxSettings           *self);
+PangoFontDescription *kgx_settings_dup_custom_font           (KgxSettings           *self);
 void                  kgx_settings_set_custom_font      (KgxSettings           *self,
                                                          PangoFontDescription  *custom_font);
 gboolean              kgx_settings_get_software_flow_control (KgxSettings      *self);
